#include <iostream>
#include <cmath>
#include <string>
#include <regex>
#include "Cmpx.hh"

const double Cmpx::kPi = std::atan(1.0)*4.0;
const double Cmpx::kEpsilon = 0.000001;

Cmpx::Cmpx(double rp, double ip) :  
  re {rp},
  im {ip}
{ }

Cmpx::Cmpx(double r) :
  Cmpx {r, 0.0}
{ }

Cmpx::Cmpx() :
  Cmpx {0.0}
{ }

// This is a "copy constructor" which we'll discuss soon.                                                                                 
Cmpx::Cmpx(const Cmpx& that) :
  re {that.re},
  im {that.im}
{ }

double Cmpx::real() {
  return re;
}

double Cmpx::imag() {
  return im;
}

Cmpx Cmpx::conjugate() {
  return Cmpx {re,-im};
}
  
double Cmpx::modulus2() {
  return times(conjugate()).re;
}

Cmpx Cmpx::reciprocal() {
  return conjugate().times(1.0 / modulus2());
}

Cmpx Cmpx::plus(Cmpx that) {
  return Cmpx {re + that.re, im + that.im};
}
  
Cmpx Cmpx::times(Cmpx that) {
  double rp = re*that.re - im*that.im;
  double ip = im*that.re + that.im*re;
  return Cmpx {rp,ip};
}

Cmpx Cmpx::times(double x) {
  return Cmpx {x*re,x*im};
}

Cmpx Cmpx::over(Cmpx that) {
  return times(that.reciprocal());
}

std::string Cmpx::to_string() {
  if (std::abs(im) < kEpsilon) {
    // Just the real part.
    return std::to_string(re);
  } else if (std::abs(re) < kEpsilon) {
    // Just the imaginary part.
    return std::to_string(im) + "i";
  } else {
    // Maybe don't include the "+".
    if (im < 0.0) {
      return std::to_string(re) + std::to_string(im) + "i";
    } else {
      return std::to_string(re) + "+" + std::to_string(im) + "i";
    }
  }    
}  
